﻿using UnityEngine;

namespace Passer.Humanoid {
    using Passer.Tracking;

    [System.Serializable]
    public class UnityXRHead : HeadSensor {
#if pUNITYXR
        public override string name {
            get { return "Unity XR"; }
        }

#if hCUSTOM
        public override HumanoidTracker tracker => humanoid.unityXR;
#endif

        protected UnityXRTracker unityXR {
            get { return humanoid.unityXR; }
        }

        #region Manage

        public override void CheckSensor(HeadTarget headTarget) {
            if (this.headTarget == null)
                this.target = headTarget;
            if (this.headTarget == null)
                return;

            if (unityXR.trackerComponent == null) {
                unityXR.CheckTracker(humanoid);
            }

            if (enabled && unityXR.trackerComponent != null && unityXR.trackerComponent.enabled) {
                if (sensorComponent == null) {
                    Vector3 position = headTarget.transform.TransformPoint(headTarget.head2eyes);
                    Quaternion rotation = headTarget.transform.rotation;
                    UnityXR unityXRtracker = unityXR.trackerComponent as UnityXR;
                    sensorComponent = unityXRtracker.GetHmd(position, rotation);
                }
                if (sensorComponent != null)
                    sensorTransform = sensorComponent.transform;

                if (!Application.isPlaying)
                    SetSensor2Target();
            }
            else {
#if UNITY_EDITOR
                if (!Application.isPlaying) {
                    if (sensorComponent != null)
                        Object.DestroyImmediate(sensorComponent.gameObject, true);
                }
#endif
                sensorComponent = null;
                sensorTransform = null;
            }
        }

        #endregion

        #region Start

        public override void Start(HumanoidControl _humanoid, Transform targetTransform) {
            base.Start(_humanoid, targetTransform);

            //tracker = headTarget.humanoid.unityXR;
            if (tracker == null || !tracker.enabled || !enabled)
                return;

            Vector3 position = headTarget.transform.TransformPoint(headTarget.head2eyes);
            Quaternion rotation = headTarget.transform.rotation;

#if hSTEAMVR && (UNITY_STANDALONE_WIN || UNITY_STANDALONE_OSX)
            if (UnityXRDevice.xrDevice == UnityXRDevice.XRDeviceType.OpenVR) {
                SteamVR steamVRtracker = unityXR.tracker as SteamVR;
                hmd = steamVRtracker.GetHmd(position, rotation);
            }
            else
#endif 
            {
                UnityXR unityXRtracker = unityXR.trackerComponent as UnityXR;
                sensorComponent = unityXRtracker.GetHmd(position, rotation);
            }
            if (sensorComponent != null)
                sensorTransform = sensorComponent.transform;

            SetSensor2Target();
            CheckSensorTransform();
            sensor2TargetPosition = -headTarget.head2eyes;

            if (sensorComponent != null)
                sensorComponent.StartComponent(unityXR.trackerComponent.transform);
        }

        #endregion

        #region Update

        protected bool calibrated = false;
        public override void Update() {
            status = Tracker.Status.Unavailable;
            if (unityXR.trackerComponent == null || !unityXR.trackerComponent.enabled || !enabled)
                return;

            if (sensorComponent == null)
                return;

            sensorComponent.UpdateComponent();
            status = sensorComponent.status;
            if (status != Tracker.Status.Tracking)
                return;

            UpdateTarget(headTarget.head.target, sensorComponent);
            UpdateNeckTargetFromHead();

            if (!calibrated) {
                if (tracker.humanoid.calibrateAtStart)
                    tracker.humanoid.Calibrate();
                calibrated = true;
            }
        }

        #endregion
#endif
    }
}